from json_database import JsonDatabaseXDG
import json


class Metric:
    def __init__(self, metric_id, metric_type, meta="{}"):
        if isinstance(meta, str):
            meta = json.loads(meta)
        self.metric_id = metric_id
        self.metric_type = metric_type
        self.meta = meta


class JsonMetricDatabase(JsonDatabaseXDG):
    def __init__(self):
        super().__init__("mycroft_metrics")

    def add_metric(self, metric_type=None, meta="{}"):
        metric_id = self.total_metrics() + 1
        metric = Metric(metric_id, metric_type, meta)
        self.add_item(metric)

    def total_metrics(self):
        return len(self)

    def __enter__(self):
        """ Context handler """
        return self

    def __exit__(self, _type, value, traceback):
        """ Commits changes and Closes the session """
        try:
            self.commit()
        except Exception as e:
            print(e)




from json_database import JsonDatabaseXDG
import json


class WakeWordRecording:
    def __init__(self, wakeword_id, transcription, path, meta="{}"):
        self.wakeword_id = wakeword_id
        self.transcription = transcription
        self.path = path
        if isinstance(meta, str):
            meta = json.loads(meta)
        self.meta = meta


class JsonWakeWordDatabase(JsonDatabaseXDG):
    def __init__(self):
        super().__init__("mycroft_wakewords")

    def add_wakeword(self, transcription, path, meta="{}"):
        wakeword_id = self.total_wakewords() + 1
        wakeword = WakeWordRecording(wakeword_id, transcription, path, meta)
        self.add_item(wakeword)

    def total_wakewords(self):
        return len(self)

    def __enter__(self):
        """ Context handler """
        return self

    def __exit__(self, _type, value, traceback):
        """ Commits changes and Closes the session """
        try:
            self.commit()
        except Exception as e:
            print(e)





from json_database import JsonDatabaseXDG


class UtteranceRecording:
    def __init__(self, utterance_id, transcription, path):
        self.utterance_id = utterance_id
        self.transcription = transcription
        self.path = path


class JsonUtteranceDatabase(JsonDatabaseXDG):
    def __init__(self):
        super().__init__("mycroft_utterances")

    def add_utterance(self, transcription, path):
        utterance_id = self.total_utterances() + 1
        utterance = UtteranceRecording(utterance_id, transcription, path)
        self.add_item(utterance)

    def total_utterances(self):
        return len(self)

    def __enter__(self):
        """ Context handler """
        return self

    def __exit__(self, _type, value, traceback):
        """ Commits changes and Closes the session """
        try:
            self.commit()
        except Exception as e:
            print(e)




from flask import request
from mock_mycroft_backend.backend.decorators import noindex
from mock_mycroft_backend.configuration import CONFIGURATION
from mock_mycroft_backend.database.wakewords import JsonWakeWordDatabase
import time
from os.path import join, isdir
from os import makedirs
import json


def get_precise_routes(app):
    @app.route('/precise/upload', methods=['POST'])
    @noindex
    def precise_upload():
        uploads = request.files
        if CONFIGURATION["record_wakewords"]:

            if not isdir(join(CONFIGURATION["data_path"], "wakewords")):
                makedirs(join(CONFIGURATION["data_path"], "wakewords"))
            name = str(time.time()).replace(".", "")
            wav_path = join(CONFIGURATION["data_path"], "wakewords",
                            name + ".wav")
            meta_path = join(CONFIGURATION["data_path"], "wakewords",
                        name + ".meta")

            for precisefile in uploads:
                fn = uploads[precisefile].filename
                if fn == 'audio':
                    uploads[precisefile].save(wav_path)
                if fn == 'metadata':
                    uploads[precisefile].save(meta_path)

            with open(meta_path) as f:
                meta = json.load(f)
            # {"name": "hey-mycroft",
            # "engine": "0f4df281688583e010c26831abdc2222",
            # "time": "1592192357852",
            # "sessionId": "7d18e208-05b5-401e-add6-ee23ae821967",
            # "accountId": "0",
            # "model": "5223842df0cdee5bca3eff8eac1b67fc"}
            with JsonWakeWordDatabase() as db:
                db.add_wakeword(meta["name"], wav_path, meta)

        uploaded = False

        return {"success": True,
                "sent_to_mycroft": uploaded,
                "saved": CONFIGURATION["record_wakewords"]}

    return app



from flask import request
from mock_mycroft_backend.session import SESSION as requests
from mock_mycroft_backend.configuration import CONFIGURATION
from mock_mycroft_backend.backend import API_VERSION
from mock_mycroft_backend.backend.decorators import noindex
from mock_mycroft_backend.utils.geolocate import geolocate, get_timezone


def get_services_routes(app):
    @app.route("/" + API_VERSION + '/geolocation', methods=['GET'])
    @noindex
    def geolocation():
        address = request.args["location"]
        data = geolocate(address)
        return {"data": {
            "city": data["city"],
            "country": data["country"],
            "latitude": float(data["lat"]),
            "longitude": float(data["lon"]),
            "region": data["region"],
            "timezone": get_timezone(float(data["lat"]), float(data["lon"]))
        }}

    @app.route("/" + API_VERSION + '/wolframAlphaSpoken', methods=['GET'])
    @noindex
    def wolfie():
        query = request.args["i"]
        units = request.args.get("units") or "metric"
        if units != "metric":
            units = "imperial"

        # not used?
        # https://products.wolframalpha.com/spoken-results-api/documentation/
        lat = str(CONFIGURATION["default_location"]["coordinate"]["latitude"])
        lon = str(CONFIGURATION["default_location"]["coordinate"]["longitude"])
        geolocation = request.args.get("geolocation") or lat + " " + lon

        url = 'http://api.wolframalpha.com/v1/spoken'
        params = {"appid": CONFIGURATION["wolfram_key"],
                  "i": query,
                  "units": units}
        answer = requests.get(url, params=params).text
        return answer

    @app.route("/" + API_VERSION + '/owm/forecast/daily', methods=['GET'])
    @noindex
    def owm_daily_forecast():
        params = dict(request.args)
        params["appid"] = CONFIGURATION["owm_key"]
        if not request.args.get("q"):
            params["lat"] = request.args.get("lat") or \
                            str(CONFIGURATION["default_location"][
                                    "coordinate"]["latitude"])
            params["lon"] = request.args.get("lon") or \
                            str(CONFIGURATION["default_location"][
                                    "coordinate"]["longitude"])
        url = "https://api.openweathermap.org/data/2.5/forecast/daily"
        return requests.get(url, params=params).json()

    @app.route("/" + API_VERSION + '/owm/forecast', methods=['GET'])
    @noindex
    def owm_3h_forecast():
        params = dict(request.args)
        params["appid"] = CONFIGURATION["owm_key"]
        if not request.args.get("q"):
            params["lat"] = request.args.get("lat") or \
                            str(CONFIGURATION["default_location"][
                                    "coordinate"]["latitude"])
            params["lon"] = request.args.get("lon") or \
                            str(CONFIGURATION["default_location"][
                                    "coordinate"]["longitude"])
        url = "https://api.openweathermap.org/data/2.5/forecast"
        return requests.get(url, params=params).json()

    @app.route("/" + API_VERSION + '/owm/weather', methods=['GET'])
    @noindex
    def owm():
        params = dict(request.args)
        params["appid"] = CONFIGURATION["owm_key"]
        if not request.args.get("q"):
            params["lat"] = request.args.get("lat") or \
                            str(CONFIGURATION["default_location"][
                                    "coordinate"]["latitude"])
            params["lon"] = request.args.get("lon") or \
                            str(CONFIGURATION["default_location"][
                                    "coordinate"]["longitude"])
        url = "https://api.openweathermap.org/data/2.5/weather"
        return requests.get(url, params=params).json()

    return app



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from os.path import exists
from json_database import JsonStorageXDG
import xdg

DEFAULT_CONFIG = {
    "stt": {"module": "google", "google": {}},
    "backend_port": 6712,
    "default_location": {
        "city": {
            "code": "Lawrence",
            "name": "Lawrence",
            "state": {
                "code": "KS",
                "name": "Kansas",
                "country": {
                    "code": "US",
                    "name": "United States"
                }
            }
        },
        "coordinate": {
            "latitude": 38.971669,
            "longitude": -95.23525
        },
        "timezone": {
            "code": "America/Chicago",
            "name": "Central Standard Time",
            "dstOffset": 3600000,
            "offset": -21600000
        }
    },
    "geolocate": False,
    "override_location": False,
    "api_version": "v1",
    "data_path": "~/.mycroft/mock_backend",
    "record_utterances": False,
    "record_wakewords": False,
    "wolfram_key": "Y7R353-9HQAAL8KKA",
    "owm_key": "28fed22898afd4717ce5a1535da1f78c"
}

CONFIGURATION = JsonStorageXDG("mycroft_backend",
                               xdg.BaseDirectory.xdg_config_home)

if not exists(CONFIGURATION.path):
    CONFIGURATION.merge(DEFAULT_CONFIG, skip_empty=False)
    CONFIGURATION.store()




# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from flask import make_response
import random
import json


def generate_code():
    k = ""
    while len(k) < 6:
        k += random.choice(["A", "B", "C", "D", "E", "F", "G", "H", "I",
                            "J", "K", "L", "M", "N", "O", "P", "Q", "R",
                            "S", "T", "U", "Y", "V", "X", "W", "Z", "0",
                            "1", "2", "3", "4", "5", "6", "7", "8", "9"])
    return k.upper()


def nice_json(arg):
    response = make_response(json.dumps(arg, sort_keys=True, indent=4))
    response.headers['Content-type'] = "application/json"
    return response



from mock_mycroft_backend.session import SESSION as requests
import geocoder
from timezonefinder import TimezoneFinder


def get_timezone(latitude, longitude):
    tf = TimezoneFinder()
    return tf.timezone_at(lng=longitude, lat=latitude)


def geolocate(address):
    data = {}
    location_data = geocoder.osm(address)
    if location_data.ok:
        location_data = location_data.json
        data["raw"] = location_data
        data["country"] = location_data.get("country")
        data["country_code"] = location_data.get("country_code")
        data["region"] = location_data.get("region")
        data["address"] = location_data.get("address")
        data["state"] = location_data.get("state")
        data["confidence"] = location_data.get("confidence")
        data["lat"] = location_data.get("lat")
        data["lon"] = location_data.get("lng")
        data["city"] = location_data.get("city")

        data["postal"] = location_data.get("postal")
        data["timezone"] = location_data.get("timezone_short")
    return data


def get_location_config(address):
    location = {
        "city": {
            "code": "",
            "name": "",
            "state": {
                "code": "",
                "name": "",
                "country": {
                    "code": "US",
                    "name": "United States"
                }
            }
        },
        "coordinate": {
            "latitude": 37.2,
            "longitude": 121.53
        },
        "timezone": {
            "dstOffset": 3600000,
            "offset": -21600000
        }
    }

    data = geolocate(address)
    location["city"]["code"] = data["city"]
    location["city"]["name"] = data["city"]
    location["city"]["state"]["name"] = data["state"]
    # TODO state code
    location["city"]["state"]["code"] = data["state"]
    location["city"]["state"]["country"]["name"] = data["country"]
    # TODO country code
    location["city"]["state"]["country"]["code"] = data["country"]
    location["coordinate"]["latitude"] = data["lat"]
    location["coordinate"]["longitude"] = data["lon"]

    timezone = get_timezone(data["lat"], data["lon"])
    location["timezone"]["name"] = data["timezone"]
    location["timezone"]["code"] = timezone

    return location


def ip_geolocate(ip):
    if ip in ["0.0.0.0", "127.0.0.1"]:
        ip = requests.get('https://api.ipify.org').text
    fields = "status,country,countryCode,region,regionName,city,lat,lon,timezone,query"
    data = requests.get("http://ip-api.com/json/" + ip,
                        params={"fields": fields}).json()
    region_data = {"code": data["region"], "name": data["regionName"],
                   "country": {
                       "code": data["countryCode"],
                       "name": data["country"]}}
    city_data = {"code": data["city"], "name": data["city"],
                 "state": region_data,
                 "region": region_data}
    timezone_data = {"code": data["timezone"],
                     "name": data["timezone"],
                     "dstOffset": 3600000,
                     "offset": -21600000}
    coordinate_data = {"latitude": float(data["lat"]),
                       "longitude": float(data["lon"])}
    return {"city": city_data,
            "coordinate": coordinate_data,
            "timezone": timezone_data}
